{-# OPTIONS -fglasgow-exts -cpp #-}
{-
     module HRayParser, which contains a parser which allows to use files containing scene descriptions
     intended for usage with the other modules in the HRay package

     author: Kenneth Hoste, 2004-2005
     part of a masters thesis at the University of Ghent, Belgium
-}

module HRayParser (RenderDescr(RenderDescr), readDescr) where

import HRayEngine (Scene(Scene), Texture(Texture), Diff(Solid,Perlin), TexturedObject,
		   Light(AmbientLight, PointLight), Camera(Camera))
import HRayPerlin (perlinSolid, perlinSemiTurbulence,perlinTurbulence,perlinFire,perlinPlasma,perlinMarble,
		   perlinMarbleBase,perlinWood)
import HRayMath (Dimension, Resolution, Object(Sphere,Plane))
import Data.Char (isSpace, isAlpha, isDigit)
#if __GLASGOW_HASKELL__ >= 503
import Data.Array
#else
import Array
#endif
#if __GLASGOW_HASKELL__ >= 503
import GHC.Exts
#else
import GlaExts
#endif

-- parser produced by Happy Version 1.17

newtype HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 = HappyAbsSyn HappyAny
#if __GLASGOW_HASKELL__ >= 607
type HappyAny = GHC.Exts.Any
#else
type HappyAny = forall a . a
#endif
happyIn4 :: t4 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn4 x = unsafeCoerce# x
{-# INLINE happyIn4 #-}
happyOut4 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t4
happyOut4 x = unsafeCoerce# x
{-# INLINE happyOut4 #-}
happyIn5 :: t5 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn5 x = unsafeCoerce# x
{-# INLINE happyIn5 #-}
happyOut5 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t5
happyOut5 x = unsafeCoerce# x
{-# INLINE happyOut5 #-}
happyIn6 :: t6 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn6 x = unsafeCoerce# x
{-# INLINE happyIn6 #-}
happyOut6 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t6
happyOut6 x = unsafeCoerce# x
{-# INLINE happyOut6 #-}
happyIn7 :: t7 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn7 x = unsafeCoerce# x
{-# INLINE happyIn7 #-}
happyOut7 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t7
happyOut7 x = unsafeCoerce# x
{-# INLINE happyOut7 #-}
happyIn8 :: t8 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn8 x = unsafeCoerce# x
{-# INLINE happyIn8 #-}
happyOut8 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t8
happyOut8 x = unsafeCoerce# x
{-# INLINE happyOut8 #-}
happyIn9 :: t9 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn9 x = unsafeCoerce# x
{-# INLINE happyIn9 #-}
happyOut9 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t9
happyOut9 x = unsafeCoerce# x
{-# INLINE happyOut9 #-}
happyIn10 :: t10 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn10 x = unsafeCoerce# x
{-# INLINE happyIn10 #-}
happyOut10 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t10
happyOut10 x = unsafeCoerce# x
{-# INLINE happyOut10 #-}
happyIn11 :: t11 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn11 x = unsafeCoerce# x
{-# INLINE happyIn11 #-}
happyOut11 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t11
happyOut11 x = unsafeCoerce# x
{-# INLINE happyOut11 #-}
happyIn12 :: t12 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn12 x = unsafeCoerce# x
{-# INLINE happyIn12 #-}
happyOut12 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t12
happyOut12 x = unsafeCoerce# x
{-# INLINE happyOut12 #-}
happyIn13 :: t13 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn13 x = unsafeCoerce# x
{-# INLINE happyIn13 #-}
happyOut13 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t13
happyOut13 x = unsafeCoerce# x
{-# INLINE happyOut13 #-}
happyIn14 :: t14 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn14 x = unsafeCoerce# x
{-# INLINE happyIn14 #-}
happyOut14 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t14
happyOut14 x = unsafeCoerce# x
{-# INLINE happyOut14 #-}
happyIn15 :: t15 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn15 x = unsafeCoerce# x
{-# INLINE happyIn15 #-}
happyOut15 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t15
happyOut15 x = unsafeCoerce# x
{-# INLINE happyOut15 #-}
happyIn16 :: t16 -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyIn16 x = unsafeCoerce# x
{-# INLINE happyIn16 #-}
happyOut16 :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> t16
happyOut16 x = unsafeCoerce# x
{-# INLINE happyOut16 #-}
happyInTok :: Token -> (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16)
happyInTok x = unsafeCoerce# x
{-# INLINE happyInTok #-}
happyOutTok :: (HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16) -> Token
happyOutTok x = unsafeCoerce# x
{-# INLINE happyOutTok #-}


happyActOffsets :: HappyAddr
happyActOffsets = HappyA# "\xde\x00\xde\x00\xda\x00\xd8\x00\xe6\x00\xdc\x00\xd6\x00\xd5\x00\xd4\x00\xe2\x00\xd0\x00\xcd\x00\xcf\x00\xd9\x00\x00\x00\xd7\x00\xca\x00\xc8\x00\xc9\x00\xd3\x00\xc4\x00\xc7\x00\x00\x00\xd2\x00\xc1\x00\xd1\x00\xc5\x00\xc3\x00\xce\x00\xc0\x00\xbd\x00\xcc\x00\xbe\x00\xbc\x00\xcb\x00\x00\x00\xbb\x00\xba\x00\xec\xff\xb9\x00\xc6\x00\x00\x00\xb6\x00\xb7\x00\xc2\x00\xb3\x00\xb2\x00\xaf\x00\xae\x00\x00\x00\x00\x00\xf5\xff\xbf\x00\xad\x00\xab\x00\xb8\x00\xaa\x00\xe8\xff\x00\x00\x00\x00\xed\xff\xb5\x00\xa8\x00\xa5\x00\x00\x00\xa3\x00\xa9\x00\xb4\x00\x9e\x00\xa1\x00\x9f\x00\xb1\x00\xb0\x00\xa7\x00\x9c\x00\x9b\x00\x00\x00\xac\x00\xa6\x00\x9a\x00\x94\x00\x93\x00\xa4\x00\xa2\x00\x9d\x00\x8e\x00\xfb\xff\x92\x00\x91\x00\x8d\x00\x99\x00\x8c\x00\x8b\x00\x8a\x00\x98\x00\x97\x00\x96\x00\x89\x00\x04\x00\x95\x00\x90\x00\x88\x00\x70\x00\x87\x00\x86\x00\x85\x00\x84\x00\x83\x00\x82\x00\x81\x00\x80\x00\x7f\x00\x00\x00\x78\x00\x77\x00\x76\x00\x75\x00\x73\x00\x00\x00\x7e\x00\x7d\x00\x7c\x00\x00\x00\x7b\x00\x7a\x00\x79\x00\x74\x00\x00\x00\x72\x00\x71\x00\x6f\x00\x6e\x00\x6d\x00\x6c\x00\x6b\x00\x6a\x00\x69\x00\x66\x00\x61\x00\x68\x00\x00\x00\x60\x00\x67\x00\x65\x00\x64\x00\x63\x00\x62\x00\x5f\x00\x5e\x00\x5d\x00\x5c\x00\x00\x00\x5b\x00\x57\x00\x56\x00\x54\x00\x51\x00\x4f\x00\x4c\x00\x47\x00\x5a\x00\x2c\x00\x59\x00\x58\x00\x55\x00\x53\x00\x52\x00\x50\x00\x4e\x00\x00\x00\x45\x00\x44\x00\x40\x00\x3f\x00\x3b\x00\x3a\x00\x38\x00\x4d\x00\x35\x00\x4b\x00\x34\x00\x4a\x00\x31\x00\x49\x00\x48\x00\x46\x00\x00\x00\x43\x00\x42\x00\x41\x00\x3e\x00\x3d\x00\x3c\x00\x29\x00\x00\x00\x39\x00\x25\x00\x24\x00\x21\x00\x00\x00\x00\x00\x37\x00\x36\x00\x33\x00\x32\x00\x00\x00\x1c\x00\x1b\x00\x19\x00\x30\x00\x2f\x00\x2e\x00\x18\x00\x16\x00\x15\x00\x2d\x00\x2b\x00\x2a\x00\x28\x00\x11\x00\x27\x00\x00\x00\x26\x00\x0a\x00\x23\x00\x00\x00\x05\x00\x22\x00\x02\x00\x20\x00\x03\x00\x1e\x00\x00\x00\x00\x00"#

happyGotoOffsets :: HappyAddr
happyGotoOffsets = HappyA# "\x1f\x00\x00\x00\x1d\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x1a\x00\x00\x00\x00\x00\x00\x00\x00\x00\x17\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x13\x00\x00\x00\x00\x00\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0f\x00\x12\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0b\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0e\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0d\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"#

happyDefActions :: HappyAddr
happyDefActions = HappyA# "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xfe\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xfd\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xeb\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xea\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xfa\xff\xe7\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe6\xff\xfc\xff\x00\x00\x00\x00\x00\x00\x00\x00\xfb\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xec\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf9\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf8\xff\x00\x00\x00\x00\x00\x00\xf3\xff\x00\x00\x00\x00\x00\x00\x00\x00\xee\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe8\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xed\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xef\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf7\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe9\xff\x00\x00\x00\x00\x00\x00\x00\x00\xf5\xff\xf6\xff\x00\x00\x00\x00\x00\x00\x00\x00\xf0\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf4\xff\x00\x00\x00\x00\x00\x00\xf2\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf1\xff"#

happyCheck :: HappyAddr
happyCheck = HappyA# "\xff\xff\x06\x00\x15\x00\x16\x00\x1c\x00\x1d\x00\x11\x00\x12\x00\x1c\x00\x1d\x00\x0f\x00\x07\x00\x08\x00\x09\x00\x0a\x00\x0b\x00\x0c\x00\x0d\x00\x0e\x00\x06\x00\x08\x00\x07\x00\x0b\x00\x05\x00\x04\x00\x09\x00\x03\x00\x0c\x00\x02\x00\x0a\x00\x01\x00\x00\x00\x02\x00\x1e\x00\x02\x00\x21\x00\x02\x00\x02\x00\x21\x00\x1d\x00\x02\x00\x02\x00\x02\x00\x01\x00\xff\xff\x02\x00\x01\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x1e\x00\x02\x00\x02\x00\x1e\x00\x21\x00\x02\x00\x02\x00\x21\x00\x02\x00\x21\x00\x21\x00\x02\x00\x02\x00\x02\x00\x1d\x00\x21\x00\x02\x00\x02\x00\x02\x00\x21\x00\x1e\x00\x02\x00\x01\x00\x01\x00\x01\x00\x01\x00\x21\x00\x1d\x00\x02\x00\x02\x00\x1d\x00\x02\x00\x1e\x00\x02\x00\x02\x00\x1e\x00\x02\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x02\x00\x21\x00\x02\x00\x02\x00\xff\xff\xff\xff\x21\x00\xff\xff\xff\xff\x21\x00\x02\x00\x21\x00\x02\x00\x02\x00\x21\x00\x02\x00\x21\x00\x21\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x02\x00\x02\x00\x02\x00\x21\x00\x21\x00\xff\xff\x02\x00\xff\xff\xff\xff\x21\x00\xff\xff\x01\x00\x21\x00\x21\x00\x21\x00\x21\x00\x21\x00\x21\x00\x1d\x00\x21\x00\x02\x00\x1e\x00\x1e\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x02\x00\x02\x00\x02\x00\x1d\x00\x1d\x00\x1d\x00\x02\x00\x1d\x00\x1d\x00\x1d\x00\x1d\x00\x02\x00\x1e\x00\x02\x00\x1e\x00\x02\x00\x02\x00\x1d\x00\x21\x00\x21\x00\x21\x00\x1d\x00\x21\x00\x1e\x00\x05\x00\x02\x00\x02\x00\x21\x00\x21\x00\x02\x00\x02\x00\x1d\x00\x10\x00\x02\x00\x21\x00\x1d\x00\x21\x00\x1d\x00\x21\x00\x01\x00\x1e\x00\x1d\x00\x01\x00\xff\xff\x1d\x00\x17\x00\x1d\x00\x02\x00\x1e\x00\x13\x00\x1e\x00\x1e\x00\x02\x00\x02\x00\x1d\x00\x02\x00\x1b\x00\x14\x00\x02\x00\x21\x00\x02\x00\x04\x00\x1d\x00\x01\x00\x1e\x00\x21\x00\x1b\x00\x03\x00\x21\x00\x21\x00\x1b\x00\x1d\x00\x1c\x00\x21\x00\x01\x00\x1c\x00\x1e\x00\x1d\x00\x01\x00\x1e\x00\xff\xff\x1b\x00\xff\xff\x18\x00\x20\x00\xff\xff\xff\xff\xff\xff\x21\x00\xff\xff\x1d\x00\x1f\x00\x19\x00\xff\xff\x1d\x00\x1a\x00\xff\xff\x22\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff"#

happyTable :: HappyAddr
happyTable = HappyA# "\x00\x00\x59\x00\x46\x00\x47\x00\x3c\x00\x3d\x00\x38\x00\x39\x00\x2b\x00\x2c\x00\x5a\x00\x69\x00\x6a\x00\x6b\x00\x6c\x00\x6d\x00\x6e\x00\x6f\x00\x70\x00\x67\x00\x41\x00\x57\x00\x3a\x00\x36\x00\x1a\x00\x29\x00\x11\x00\x39\x00\x0b\x00\x26\x00\x04\x00\x03\x00\xe9\x00\xe8\x00\xe7\x00\xe6\x00\xe5\x00\xe3\x00\xe4\x00\xe1\x00\xe2\x00\xde\x00\xe0\x00\xdb\x00\x00\x00\xdc\x00\xdd\x00\xdf\x00\xd5\x00\xd6\x00\xd7\x00\xd8\x00\xce\x00\xcf\x00\xda\x00\xd9\x00\xd0\x00\xd1\x00\xd2\x00\xcd\x00\xd3\x00\xd4\x00\xc4\x00\xc5\x00\xc6\x00\xcb\x00\xca\x00\xc7\x00\xc8\x00\xc9\x00\xcc\x00\xc3\x00\xbb\x00\xbc\x00\xbd\x00\xbf\x00\xc1\x00\xb2\x00\xbe\x00\xb3\x00\xab\x00\xc0\x00\xac\x00\xc2\x00\xad\x00\xae\x00\xb4\x00\xaf\x00\xb5\x00\xb6\x00\xb0\x00\xb1\x00\xa2\x00\xb7\x00\xb8\x00\x99\x00\x9a\x00\x9b\x00\xb9\x00\xba\x00\x9c\x00\x9d\x00\x9e\x00\x9f\x00\xa3\x00\xa0\x00\x8e\x00\x00\x00\x00\x00\xa4\x00\x00\x00\x00\x00\xa5\x00\x97\x00\xa6\x00\x83\x00\x84\x00\xa7\x00\x85\x00\xa8\x00\xa9\x00\xaa\x00\x98\x00\x86\x00\x87\x00\x88\x00\x89\x00\x8a\x00\x8b\x00\xa1\x00\x8f\x00\x00\x00\x7b\x00\x00\x00\x00\x00\x90\x00\x00\x00\x82\x00\x91\x00\x92\x00\x93\x00\x94\x00\x95\x00\x96\x00\x8c\x00\x81\x00\x66\x00\x8d\x00\x75\x00\x76\x00\x77\x00\x67\x00\x72\x00\x73\x00\x74\x00\x62\x00\x78\x00\x79\x00\x7a\x00\x5c\x00\x7c\x00\x7d\x00\x7e\x00\x7f\x00\x5d\x00\x80\x00\x5e\x00\x71\x00\x56\x00\x50\x00\x63\x00\x5f\x00\x60\x00\x61\x00\x64\x00\x5b\x00\x65\x00\x57\x00\x51\x00\x52\x00\x53\x00\x54\x00\x4b\x00\x45\x00\x4e\x00\x4c\x00\x3f\x00\x55\x00\x48\x00\x4f\x00\x49\x00\x4a\x00\x36\x00\x4d\x00\x43\x00\x2e\x00\x00\x00\x44\x00\x33\x00\x3e\x00\x31\x00\x40\x00\x2f\x00\x41\x00\x32\x00\x28\x00\x25\x00\x34\x00\x22\x00\x30\x00\x24\x00\x1f\x00\x35\x00\x19\x00\x1c\x00\x2d\x00\x11\x00\x26\x00\x29\x00\x21\x00\x13\x00\x23\x00\x20\x00\x18\x00\x1d\x00\x1e\x00\x1a\x00\x0b\x00\x15\x00\x17\x00\x14\x00\x08\x00\x16\x00\x00\x00\x0e\x00\x00\x00\x0d\x00\x0f\x00\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x0a\x00\x09\x00\x07\x00\x00\x00\x06\x00\x03\x00\x00\x00\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"#

happyReduceArr = array (1, 25) [
	(1 , happyReduce_1),
	(2 , happyReduce_2),
	(3 , happyReduce_3),
	(4 , happyReduce_4),
	(5 , happyReduce_5),
	(6 , happyReduce_6),
	(7 , happyReduce_7),
	(8 , happyReduce_8),
	(9 , happyReduce_9),
	(10 , happyReduce_10),
	(11 , happyReduce_11),
	(12 , happyReduce_12),
	(13 , happyReduce_13),
	(14 , happyReduce_14),
	(15 , happyReduce_15),
	(16 , happyReduce_16),
	(17 , happyReduce_17),
	(18 , happyReduce_18),
	(19 , happyReduce_19),
	(20 , happyReduce_20),
	(21 , happyReduce_21),
	(22 , happyReduce_22),
	(23 , happyReduce_23),
	(24 , happyReduce_24),
	(25 , happyReduce_25)
	]

happy_n_terms = 35 :: Int
happy_n_nonterms = 13 :: Int

happyReduce_1 = happyReduce 6# 0# happyReduction_1
happyReduction_1 (happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOut5 happy_x_2 of { happy_var_2 -> 
	case happyOutTok happy_x_3 of { (TokenInt happy_var_3) -> 
	case happyOut6 happy_x_5 of { happy_var_5 -> 
	happyIn4
		 (RenderDescr happy_var_2 happy_var_3 happy_var_5
	) `HappyStk` happyRest}}}

happyReduce_2 = happyReduce 8# 1# happyReduction_2
happyReduction_2 (happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_4 of { (TokenInt happy_var_4) -> 
	case happyOutTok happy_x_6 of { (TokenInt happy_var_6) -> 
	happyIn5
		 ((happy_var_4,happy_var_6)
	) `HappyStk` happyRest}}

happyReduce_3 = happyReduce 15# 2# happyReduction_3
happyReduction_3 (happy_x_15 `HappyStk`
	happy_x_14 `HappyStk`
	happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOut7 happy_x_3 of { happy_var_3 -> 
	case happyOut8 happy_x_6 of { happy_var_6 -> 
	case happyOut14 happy_x_10 of { happy_var_10 -> 
	case happyOut16 happy_x_14 of { happy_var_14 -> 
	happyIn6
		 (Scene happy_var_3 happy_var_6 happy_var_10 happy_var_14
	) `HappyStk` happyRest}}}}

happyReduce_4 = happyReduce 13# 3# happyReduction_4
happyReduction_4 (happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_10 of { (TokenInt happy_var_10) -> 
	case happyOutTok happy_x_12 of { (TokenInt happy_var_12) -> 
	happyIn7
		 (Camera (happy_var_3,happy_var_5,happy_var_7) (happy_var_10,happy_var_12)
	) `HappyStk` happyRest}}}}}

happyReduce_5 = happyReduce 8# 4# happyReduction_5
happyReduction_5 (happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	happyIn8
		 ((happy_var_3,happy_var_5,happy_var_7)
	) `HappyStk` happyRest}}}

happyReduce_6 = happyReduce 9# 5# happyReduction_6
happyReduction_6 (happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_2 of { (TokenDouble happy_var_2) -> 
	case happyOutTok happy_x_4 of { (TokenDouble happy_var_4) -> 
	case happyOutTok happy_x_6 of { (TokenDouble happy_var_6) -> 
	case happyOutTok happy_x_8 of { (TokenDouble happy_var_8) -> 
	happyIn9
		 (Sphere happy_var_2 (happy_var_4,happy_var_6,happy_var_8)
	) `HappyStk` happyRest}}}}

happyReduce_7 = happyReduce 10# 5# happyReduction_7
happyReduction_7 (happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenDouble happy_var_9) -> 
	happyIn9
		 (Plane (happy_var_3,happy_var_5,happy_var_7,happy_var_9)
	) `HappyStk` happyRest}}}}

happyReduce_8 = happyReduce 9# 6# happyReduction_8
happyReduction_8 (happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenDouble happy_var_9) -> 
	happyIn10
		 (perlinSolid (happy_var_3,happy_var_5,happy_var_7) happy_var_9
	) `HappyStk` happyRest}}}}

happyReduce_9 = happyReduce 10# 6# happyReduction_9
happyReduction_9 (happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenInt happy_var_9) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	happyIn10
		 (perlinSemiTurbulence (happy_var_3,happy_var_5,happy_var_7) happy_var_9 happy_var_10
	) `HappyStk` happyRest}}}}}

happyReduce_10 = happyReduce 10# 6# happyReduction_10
happyReduction_10 (happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenInt happy_var_9) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	happyIn10
		 (perlinTurbulence (happy_var_3,happy_var_5,happy_var_7) happy_var_9 happy_var_10
	) `HappyStk` happyRest}}}}}

happyReduce_11 = happyReduce 17# 6# happyReduction_11
happyReduction_11 (happy_x_17 `HappyStk`
	happy_x_16 `HappyStk`
	happy_x_15 `HappyStk`
	happy_x_14 `HappyStk`
	happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	case happyOutTok happy_x_12 of { (TokenDouble happy_var_12) -> 
	case happyOutTok happy_x_14 of { (TokenDouble happy_var_14) -> 
	case happyOutTok happy_x_16 of { (TokenInt happy_var_16) -> 
	case happyOutTok happy_x_17 of { (TokenDouble happy_var_17) -> 
	happyIn10
		 (perlinFire (happy_var_3,happy_var_5,happy_var_7) (happy_var_10,happy_var_12,happy_var_14)
										            happy_var_16 happy_var_17
	) `HappyStk` happyRest}}}}}}}}

happyReduce_12 = happySpecReduce_2  6# happyReduction_12
happyReduction_12 happy_x_2
	happy_x_1
	 =  case happyOutTok happy_x_2 of { (TokenDouble happy_var_2) -> 
	happyIn10
		 (perlinPlasma happy_var_2
	)}

happyReduce_13 = happyReduce 18# 6# happyReduction_13
happyReduction_13 (happy_x_18 `HappyStk`
	happy_x_17 `HappyStk`
	happy_x_16 `HappyStk`
	happy_x_15 `HappyStk`
	happy_x_14 `HappyStk`
	happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenInt happy_var_9) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	case happyOutTok happy_x_12 of { (TokenDouble happy_var_12) -> 
	case happyOutTok happy_x_14 of { (TokenDouble happy_var_14) -> 
	case happyOutTok happy_x_16 of { (TokenDouble happy_var_16) -> 
	case happyOutTok happy_x_18 of { (TokenDouble happy_var_18) -> 
	happyIn10
		 (perlinMarble (happy_var_3,happy_var_5,happy_var_7) happy_var_9 happy_var_10
										              (happy_var_12,happy_var_14,happy_var_16) happy_var_18
	) `HappyStk` happyRest}}}}}}}}}

happyReduce_14 = happyReduce 25# 6# happyReduction_14
happyReduction_14 (happy_x_25 `HappyStk`
	happy_x_24 `HappyStk`
	happy_x_23 `HappyStk`
	happy_x_22 `HappyStk`
	happy_x_21 `HappyStk`
	happy_x_20 `HappyStk`
	happy_x_19 `HappyStk`
	happy_x_18 `HappyStk`
	happy_x_17 `HappyStk`
	happy_x_16 `HappyStk`
	happy_x_15 `HappyStk`
	happy_x_14 `HappyStk`
	happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	case happyOutTok happy_x_12 of { (TokenDouble happy_var_12) -> 
	case happyOutTok happy_x_14 of { (TokenDouble happy_var_14) -> 
	case happyOutTok happy_x_16 of { (TokenInt happy_var_16) -> 
	case happyOutTok happy_x_17 of { (TokenDouble happy_var_17) -> 
	case happyOutTok happy_x_19 of { (TokenDouble happy_var_19) -> 
	case happyOutTok happy_x_21 of { (TokenDouble happy_var_21) -> 
	case happyOutTok happy_x_23 of { (TokenDouble happy_var_23) -> 
	case happyOutTok happy_x_25 of { (TokenDouble happy_var_25) -> 
	happyIn10
		 (perlinMarbleBase (happy_var_3,happy_var_5,happy_var_7)
										                  (happy_var_10,happy_var_12,happy_var_14)
                                                                                                  happy_var_16 happy_var_17
										                  (happy_var_19,happy_var_21,happy_var_23) happy_var_25
	) `HappyStk` happyRest}}}}}}}}}}}}

happyReduce_15 = happyReduce 12# 6# happyReduction_15
happyReduction_15 (happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_9 of { (TokenInt happy_var_9) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	case happyOutTok happy_x_11 of { (TokenDouble happy_var_11) -> 
	case happyOutTok happy_x_12 of { (TokenDouble happy_var_12) -> 
	happyIn10
		 (perlinWood (happy_var_3,happy_var_5,happy_var_7) happy_var_9 happy_var_10 happy_var_11 happy_var_12
	) `HappyStk` happyRest}}}}}}}

happyReduce_16 = happyReduce 8# 7# happyReduction_16
happyReduction_16 (happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_3 of { (TokenDouble happy_var_3) -> 
	case happyOutTok happy_x_5 of { (TokenDouble happy_var_5) -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	happyIn11
		 (Solid (happy_var_3,happy_var_5,happy_var_7)
	) `HappyStk` happyRest}}}

happyReduce_17 = happyReduce 4# 7# happyReduction_17
happyReduction_17 (happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOut10 happy_x_3 of { happy_var_3 -> 
	happyIn11
		 (Perlin happy_var_3
	) `HappyStk` happyRest}

happyReduce_18 = happyReduce 11# 8# happyReduction_18
happyReduction_18 (happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOut11 happy_x_5 of { happy_var_5 -> 
	case happyOutTok happy_x_7 of { (TokenDouble happy_var_7) -> 
	case happyOutTok happy_x_8 of { (TokenInt happy_var_8) -> 
	case happyOutTok happy_x_9 of { (TokenDouble happy_var_9) -> 
	case happyOutTok happy_x_10 of { (TokenDouble happy_var_10) -> 
	happyIn12
		 (Texture happy_var_5 happy_var_7 happy_var_8 happy_var_9 happy_var_10
	) `HappyStk` happyRest}}}}}

happyReduce_19 = happyReduce 7# 9# happyReduction_19
happyReduction_19 (happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOut9 happy_x_4 of { happy_var_4 -> 
	case happyOut12 happy_x_6 of { happy_var_6 -> 
	happyIn13
		 ((happy_var_4,happy_var_6)
	) `HappyStk` happyRest}}

happyReduce_20 = happySpecReduce_0  10# happyReduction_20
happyReduction_20  =  happyIn14
		 ([]
	)

happyReduce_21 = happySpecReduce_2  10# happyReduction_21
happyReduction_21 happy_x_2
	happy_x_1
	 =  case happyOut14 happy_x_1 of { happy_var_1 -> 
	case happyOut13 happy_x_2 of { happy_var_2 -> 
	happyIn14
		 (happy_var_2 : happy_var_1
	)}}

happyReduce_22 = happyReduce 17# 11# happyReduction_22
happyReduction_22 (happy_x_17 `HappyStk`
	happy_x_16 `HappyStk`
	happy_x_15 `HappyStk`
	happy_x_14 `HappyStk`
	happy_x_13 `HappyStk`
	happy_x_12 `HappyStk`
	happy_x_11 `HappyStk`
	happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_4 of { (TokenDouble happy_var_4) -> 
	case happyOutTok happy_x_6 of { (TokenDouble happy_var_6) -> 
	case happyOutTok happy_x_8 of { (TokenDouble happy_var_8) -> 
	case happyOutTok happy_x_11 of { (TokenDouble happy_var_11) -> 
	case happyOutTok happy_x_13 of { (TokenDouble happy_var_13) -> 
	case happyOutTok happy_x_15 of { (TokenDouble happy_var_15) -> 
	happyIn15
		 (PointLight (happy_var_4,happy_var_6,happy_var_8) (happy_var_11,happy_var_13,happy_var_15)
	) `HappyStk` happyRest}}}}}}

happyReduce_23 = happyReduce 10# 11# happyReduction_23
happyReduction_23 (happy_x_10 `HappyStk`
	happy_x_9 `HappyStk`
	happy_x_8 `HappyStk`
	happy_x_7 `HappyStk`
	happy_x_6 `HappyStk`
	happy_x_5 `HappyStk`
	happy_x_4 `HappyStk`
	happy_x_3 `HappyStk`
	happy_x_2 `HappyStk`
	happy_x_1 `HappyStk`
	happyRest)
	 = case happyOutTok happy_x_4 of { (TokenDouble happy_var_4) -> 
	case happyOutTok happy_x_6 of { (TokenDouble happy_var_6) -> 
	case happyOutTok happy_x_8 of { (TokenDouble happy_var_8) -> 
	happyIn15
		 (AmbientLight (happy_var_4,happy_var_6,happy_var_8)
	) `HappyStk` happyRest}}}

happyReduce_24 = happySpecReduce_0  12# happyReduction_24
happyReduction_24  =  happyIn16
		 ([]
	)

happyReduce_25 = happySpecReduce_2  12# happyReduction_25
happyReduction_25 happy_x_2
	happy_x_1
	 =  case happyOut16 happy_x_1 of { happy_var_1 -> 
	case happyOut15 happy_x_2 of { happy_var_2 -> 
	happyIn16
		 (happy_var_2 : happy_var_1
	)}}

happyNewToken action sts stk [] =
	happyDoAction 34# notHappyAtAll action sts stk []

happyNewToken action sts stk (tk:tks) =
	let cont i = happyDoAction i tk action sts stk tks in
	case tk of {
	TokenInt happy_dollar_dollar -> cont 1#;
	TokenDouble happy_dollar_dollar -> cont 2#;
	TokenCamera -> cont 3#;
	TokenBackground -> cont 4#;
	TokenDiff -> cont 5#;
	TokenSolid -> cont 6#;
	TokenPerlinSolid -> cont 7#;
	TokenPerlinSemiTurb -> cont 8#;
	TokenPerlinTurb -> cont 9#;
	TokenPerlinFire -> cont 10#;
	TokenPerlinPlasma -> cont 11#;
	TokenPerlinMarble -> cont 12#;
	TokenPerlinMarbleBase -> cont 13#;
	TokenPerlinWood -> cont 14#;
	TokenPerlin -> cont 15#;
	TokenTexture -> cont 16#;
	TokenSphere -> cont 17#;
	TokenPlane -> cont 18#;
	TokenTexturedObject -> cont 19#;
	TokenObjects -> cont 20#;
	TokenPointLight -> cont 21#;
	TokenAmbientLight -> cont 22#;
	TokenLights -> cont 23#;
	TokenScene -> cont 24#;
	TokenResolution -> cont 25#;
	TokenRenderDescr -> cont 26#;
	TokenOpenAcc -> cont 27#;
	TokenCloseAcc -> cont 28#;
	TokenOpenBrack -> cont 29#;
	TokenCloseBrack -> cont 30#;
	TokenOpenHook -> cont 31#;
	TokenCloseHook -> cont 32#;
	TokenComma -> cont 33#;
	_ -> happyError' (tk:tks)
	}

happyError_ tk tks = happyError' (tk:tks)

newtype HappyIdentity a = HappyIdentity a
happyIdentity = HappyIdentity
happyRunIdentity (HappyIdentity a) = a

instance Monad HappyIdentity where
    return = HappyIdentity
    (HappyIdentity p) >>= q = q p

happyThen :: () => HappyIdentity a -> (a -> HappyIdentity b) -> HappyIdentity b
happyThen = (>>=)
happyReturn :: () => a -> HappyIdentity a
happyReturn = (return)
happyThen1 m k tks = (>>=) m (\a -> k a tks)
happyReturn1 :: () => a -> b -> HappyIdentity a
happyReturn1 = \a tks -> (return) a
happyError' :: () => [Token] -> HappyIdentity a
happyError' = HappyIdentity . happyError

parseScene tks = happyRunIdentity happySomeParser where
  happySomeParser = happyThen (happyParse 0# tks) (\x -> happyReturn (happyOut4 x))

happySeq = happyDontSeq


-- representation of a full description of scene which should be rendered
data RenderDescr = RenderDescr Resolution Int Scene


readDescr :: String -> RenderDescr
readDescr = parseScene.lexer

happyError :: [Token] -> a
happyError _ = error "Parse error ! ! !"

data Token
       = TokenInt Int
       | TokenDouble Double
       | TokenCamera
       | TokenBackground
       | TokenDiff
       | TokenSolid
       | TokenPerlinSolid
       | TokenPerlinSemiTurb
       | TokenPerlinTurb
       | TokenPerlinFire
       | TokenPerlinPlasma
       | TokenPerlinMarble
       | TokenPerlinMarbleBase
       | TokenPerlinWood
       | TokenPerlin
       | TokenTexture
       | TokenSphere
       | TokenPlane
       | TokenObjType
       | TokenTexturedObject
       | TokenObjects
       | TokenPointLight
       | TokenAmbientLight
       | TokenLight
       | TokenLights
       | TokenScene
       | TokenResolution
       | TokenRenderDescr
       | TokenOpenAcc
       | TokenCloseAcc
       | TokenOpenBrack
       | TokenCloseBrack
       | TokenOpenHook
       | TokenCloseHook
       | TokenComma

lexer :: String -> [Token]
lexer [] = []
lexer (c:cs)
      | isSpace c = lexer cs
      | isAlpha c = lexVar (c:cs)
      | isDigit c = lexNum (c:cs) 1
lexer ('{':cs) = TokenOpenAcc : lexer cs
lexer ('}':cs) = TokenCloseAcc : lexer cs
lexer ('(':cs) = TokenOpenBrack : lexer cs
lexer (')':cs) = TokenCloseBrack : lexer cs
lexer ('[':cs) = TokenOpenHook : lexer cs
lexer (']':cs) = TokenCloseHook : lexer cs
lexer (',':cs) = TokenComma : lexer cs
lexer ('-':cs) = lexNum cs (-1)

lexNum cs mul
          | (r == '.')  = TokenDouble (mul * (read (num++[r]++num2) :: Double)) : lexer rest2
          | otherwise = TokenInt (round (mul * (read num))) : lexer (r:rest)
          where (num,(r:rest)) = span isDigit cs
	        (num2,rest2)   = span isDigit rest

lexVar cs =
   case span isAlpha cs of
      ("camera",rest)            -> TokenCamera : lexer rest
      ("background",rest)        -> TokenBackground : lexer rest
      ("diff",rest)              -> TokenDiff : lexer rest
      ("solid",rest)             -> TokenSolid : lexer rest
      ("perlinSolid",rest)       -> TokenPerlinSolid : lexer rest
      ("perlinSemiTurb",rest)    -> TokenPerlinSemiTurb : lexer rest
      ("perlinTurb",rest)        -> TokenPerlinTurb : lexer rest
      ("perlinFire",rest)        -> TokenPerlinFire : lexer rest
      ("perlinPlasma",rest)      -> TokenPerlinPlasma : lexer rest
      ("perlinMarble",rest)      -> TokenPerlinMarble : lexer rest
      ("perlinMarbleBase",rest)  -> TokenPerlinMarbleBase : lexer rest
      ("perlinWood",rest)        -> TokenPerlinWood : lexer rest
      ("perlin",rest)            -> TokenPerlin : lexer rest
      ("texture",rest)           -> TokenTexture : lexer rest
      ("sphere",rest)            -> TokenSphere : lexer rest
      ("plane",rest)             -> TokenPlane : lexer rest
      ("object",rest)            -> TokenTexturedObject : lexer rest
      ("objects",rest)           -> TokenObjects : lexer rest
      ("pointLight",rest)        -> TokenPointLight : lexer rest
      ("ambientLight",rest)      -> TokenAmbientLight : lexer rest
      ("light",rest)             -> TokenLight : lexer rest
      ("lights",rest)            -> TokenLights : lexer rest
      ("scene",rest)             -> TokenScene : lexer rest
      ("resolution",rest)        -> TokenResolution : lexer rest
      ("renderDescr",rest)       -> TokenRenderDescr : lexer rest
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "<built-in>" #-}
{-# LINE 1 "<command-line>" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- Id: GenericTemplate.hs,v 1.26 2005/01/14 14:47:22 simonmar Exp 

{-# LINE 28 "templates/GenericTemplate.hs" #-}


data Happy_IntList = HappyCons Int# Happy_IntList





{-# LINE 49 "templates/GenericTemplate.hs" #-}

{-# LINE 59 "templates/GenericTemplate.hs" #-}

{-# LINE 68 "templates/GenericTemplate.hs" #-}

infixr 9 `HappyStk`
data HappyStk a = HappyStk a (HappyStk a)

-----------------------------------------------------------------------------
-- starting the parse

happyParse start_state = happyNewToken start_state notHappyAtAll notHappyAtAll

-----------------------------------------------------------------------------
-- Accepting the parse

-- If the current token is 0#, it means we've just accepted a partial
-- parse (a %partial parser).  We must ignore the saved token on the top of
-- the stack in this case.
happyAccept 0# tk st sts (_ `HappyStk` ans `HappyStk` _) =
	happyReturn1 ans
happyAccept j tk st sts (HappyStk ans _) = 
	(happyTcHack j (happyTcHack st)) (happyReturn1 ans)

-----------------------------------------------------------------------------
-- Arrays only: do the next action



happyDoAction i tk st
	= {- nothing -}


	  case action of
		0#		  -> {- nothing -}
				     happyFail i tk st
		-1# 	  -> {- nothing -}
				     happyAccept i tk st
		n | (n <# (0# :: Int#)) -> {- nothing -}

				     (happyReduceArr ! rule) i tk st
				     where rule = (I# ((negateInt# ((n +# (1# :: Int#))))))
		n		  -> {- nothing -}


				     happyShift new_state i tk st
				     where new_state = (n -# (1# :: Int#))
   where off    = indexShortOffAddr happyActOffsets st
	 off_i  = (off +# i)
	 check  = if (off_i >=# (0# :: Int#))
			then (indexShortOffAddr happyCheck off_i ==#  i)
			else False
 	 action | check     = indexShortOffAddr happyTable off_i
		| otherwise = indexShortOffAddr happyDefActions st

{-# LINE 127 "templates/GenericTemplate.hs" #-}


indexShortOffAddr (HappyA# arr) off =
#if __GLASGOW_HASKELL__ > 500
	narrow16Int# i
#elif __GLASGOW_HASKELL__ == 500
	intToInt16# i
#else
	(i `iShiftL#` 16#) `iShiftRA#` 16#
#endif
  where
#if __GLASGOW_HASKELL__ >= 503
	i = word2Int# ((high `uncheckedShiftL#` 8#) `or#` low)
#else
	i = word2Int# ((high `shiftL#` 8#) `or#` low)
#endif
	high = int2Word# (ord# (indexCharOffAddr# arr (off' +# 1#)))
	low  = int2Word# (ord# (indexCharOffAddr# arr off'))
	off' = off *# 2#





data HappyAddr = HappyA# Addr#




-----------------------------------------------------------------------------
-- HappyState data type (not arrays)

{-# LINE 170 "templates/GenericTemplate.hs" #-}

-----------------------------------------------------------------------------
-- Shifting a token

happyShift new_state 0# tk st sts stk@(x `HappyStk` _) =
     let i = (case unsafeCoerce# x of { (I# (i)) -> i }) in
--     trace "shifting the error token" $
     happyDoAction i tk new_state (HappyCons (st) (sts)) (stk)

happyShift new_state i tk st sts stk =
     happyNewToken new_state (HappyCons (st) (sts)) ((happyInTok (tk))`HappyStk`stk)

-- happyReduce is specialised for the common cases.

happySpecReduce_0 i fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happySpecReduce_0 nt fn j tk st@((action)) sts stk
     = happyGoto nt j tk st (HappyCons (st) (sts)) (fn `HappyStk` stk)

happySpecReduce_1 i fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happySpecReduce_1 nt fn j tk _ sts@((HappyCons (st@(action)) (_))) (v1`HappyStk`stk')
     = let r = fn v1 in
       happySeq r (happyGoto nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_2 i fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happySpecReduce_2 nt fn j tk _ (HappyCons (_) (sts@((HappyCons (st@(action)) (_))))) (v1`HappyStk`v2`HappyStk`stk')
     = let r = fn v1 v2 in
       happySeq r (happyGoto nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_3 i fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happySpecReduce_3 nt fn j tk _ (HappyCons (_) ((HappyCons (_) (sts@((HappyCons (st@(action)) (_))))))) (v1`HappyStk`v2`HappyStk`v3`HappyStk`stk')
     = let r = fn v1 v2 v3 in
       happySeq r (happyGoto nt j tk st sts (r `HappyStk` stk'))

happyReduce k i fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happyReduce k nt fn j tk st sts stk
     = case happyDrop (k -# (1# :: Int#)) sts of
	 sts1@((HappyCons (st1@(action)) (_))) ->
        	let r = fn stk in  -- it doesn't hurt to always seq here...
       		happyDoSeq r (happyGoto nt j tk st1 sts1 r)

happyMonadReduce k nt fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happyMonadReduce k nt fn j tk st sts stk =
        happyThen1 (fn stk tk) (\r -> happyGoto nt j tk st1 sts1 (r `HappyStk` drop_stk))
       where sts1@((HappyCons (st1@(action)) (_))) = happyDrop k (HappyCons (st) (sts))
             drop_stk = happyDropStk k stk

happyMonad2Reduce k nt fn 0# tk st sts stk
     = happyFail 0# tk st sts stk
happyMonad2Reduce k nt fn j tk st sts stk =
       happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
       where sts1@((HappyCons (st1@(action)) (_))) = happyDrop k (HappyCons (st) (sts))
             drop_stk = happyDropStk k stk

             off    = indexShortOffAddr happyGotoOffsets st1
             off_i  = (off +# nt)
             new_state = indexShortOffAddr happyTable off_i




happyDrop 0# l = l
happyDrop n (HappyCons (_) (t)) = happyDrop (n -# (1# :: Int#)) t

happyDropStk 0# l = l
happyDropStk n (x `HappyStk` xs) = happyDropStk (n -# (1#::Int#)) xs

-----------------------------------------------------------------------------
-- Moving to a new state after a reduction


happyGoto nt j tk st = 
   {- nothing -}
   happyDoAction j tk new_state
   where off    = indexShortOffAddr happyGotoOffsets st
	 off_i  = (off +# nt)
 	 new_state = indexShortOffAddr happyTable off_i




-----------------------------------------------------------------------------
-- Error recovery (0# is the error token)

-- parse error if we are in recovery and we fail again
happyFail  0# tk old_st _ stk =
--	trace "failing" $ 
    	happyError_ tk

{-  We don't need state discarding for our restricted implementation of
    "error".  In fact, it can cause some bogus parses, so I've disabled it
    for now --SDM

-- discard a state
happyFail  0# tk old_st (HappyCons ((action)) (sts)) 
						(saved_tok `HappyStk` _ `HappyStk` stk) =
--	trace ("discarding state, depth " ++ show (length stk))  $
	happyDoAction 0# tk action sts ((saved_tok`HappyStk`stk))
-}

-- Enter error recovery: generate an error token,
--                       save the old token and carry on.
happyFail  i tk (action) sts stk =
--      trace "entering error recovery" $
	happyDoAction 0# tk action sts ( (unsafeCoerce# (I# (i))) `HappyStk` stk)

-- Internal happy errors:

notHappyAtAll = error "Internal Happy error\n"

-----------------------------------------------------------------------------
-- Hack to get the typechecker to accept our action functions


happyTcHack :: Int# -> a -> a
happyTcHack x y = y
{-# INLINE happyTcHack #-}


-----------------------------------------------------------------------------
-- Seq-ing.  If the --strict flag is given, then Happy emits 
--	happySeq = happyDoSeq
-- otherwise it emits
-- 	happySeq = happyDontSeq

happyDoSeq, happyDontSeq :: a -> b -> b
happyDoSeq   a b = a `seq` b
happyDontSeq a b = b

-----------------------------------------------------------------------------
-- Don't inline any functions from the template.  GHC has a nasty habit
-- of deciding to inline happyGoto everywhere, which increases the size of
-- the generated parser quite a bit.


{-# NOINLINE happyDoAction #-}
{-# NOINLINE happyTable #-}
{-# NOINLINE happyCheck #-}
{-# NOINLINE happyActOffsets #-}
{-# NOINLINE happyGotoOffsets #-}
{-# NOINLINE happyDefActions #-}

{-# NOINLINE happyShift #-}
{-# NOINLINE happySpecReduce_0 #-}
{-# NOINLINE happySpecReduce_1 #-}
{-# NOINLINE happySpecReduce_2 #-}
{-# NOINLINE happySpecReduce_3 #-}
{-# NOINLINE happyReduce #-}
{-# NOINLINE happyMonadReduce #-}
{-# NOINLINE happyGoto #-}
{-# NOINLINE happyFail #-}

-- end of Happy Template.
